@@ -6,15 +6,16 @@ import time
 import requests
 from requests import Session
 
-from twitter_crawler.error import TwitterError
+from error import TwitterError
+
+PUBLIC_TOKEN = "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
 
 
 def get_guest_token(proxies, retries=3):
     headers = {
         "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 "
                       "Safari/537.36",
-        "Authorization": "Bearer AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
-                         "%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA",
+        "Authorization": f"Bearer {PUBLIC_TOKEN}",
     }
     token = None
     while retries > 0:
@@ -32,6 +33,7 @@ def get_guest_token(proxies, retries=3):
     return token
 
 
+# 该接口时而可用，时而不可用，被替换成新版本
 def get_guest_token_v0(proxies, retries=3):
     headers = {
         "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 "
@@ -76,10 +78,6 @@ class RateLimit:
 
 
 class Api:
-    DEFAULT_AUTHORIZATION = (
-        "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
-        "%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA "
-    )
     URI = "https://api.twitter.com"
 
     def __init__(self, proxies=None):
@@ -194,7 +192,7 @@ class Api:
             "x-guest-token": self.guest_token,
             "x-twitter-active-user": "yes",
             "Origin": "https://twitter.com",
-            "Authorization": f"Bearer {self.DEFAULT_AUTHORIZATION}",
+            "Authorization": f"Bearer {PUBLIC_TOKEN}",
             "Referer": "https://twitter.com/",
         }
         return headers
@@ -216,8 +214,8 @@ class Api:
             "include_tweet_replies": True,
             "tweet_mode": "extended",  # 帖子返回风格
             "include_card_uri": True,
-            "include_rts": True,  # 是否包含转发贴
-            "exclude_replies": True,  # 是否拦截回复贴（true是不带，false是带）
+            "include_rts": False,  # 是否包含转发贴
+            "exclude_replies": False,  # 是否拦截回复贴（true是不带，false是带）
             "trim_user": False,  # 是否只返回用户的精简数据
         }
         return {**params_public, **params}
@@ -274,10 +272,10 @@ class Api:
             data = self.req_twitter(
                 url=self.URI + "/1.1/statuses/user_timeline.json", params=params
             )
+            tweets = tweets + data
             # 如果没有新数据了 直接返回
             if len(data) <= 1:
                 break
-            tweets = tweets + data
             # 获取一下下一页的游标
             max_id = data[-1]["id_str"]
 
@@ -443,13 +441,14 @@ class Api:
         -------
         list
             帖子回复列表
+            作者列表
 
         References
         ----------
         https://tweetdeck.twitter.com/
         """
         comments = {}
-        users = {}
+        authers = {}
         tweet_ids = set()
         user_ids = set()
         params = self.get_params({})
@@ -460,6 +459,7 @@ class Api:
                 url=self.URI + f"/2/timeline/conversation/{post_id}.json", params=params
             )
             tweets = data["globalObjects"]["tweets"]
+            users = data["globalObjects"]["users"]
             # 如果没有新数据了 直接返回
             if len(tweets) <= 1:
                 break
@@ -474,8 +474,8 @@ class Api:
             for user_id, user in users.items():
                 if user_id not in user_ids:
                     user_ids.add(user_id)
-                    users[user_id] = user
-            users.update(data["globalObjects"]["users"])
+                    authers[user_id] = user
+
             # 获取一下下一页的游标
             try:
                 cursor = data["timeline"]["instructions"][0]["addEntries"]["entries"][-1]["content"]["operation"][
@@ -486,7 +486,7 @@ class Api:
             if len(comments) > count:
                 break
 
-        return comments, users
+        return comments, authers
 
     # 关键词搜索帖子
     def search_tweets(
@@ -526,7 +526,7 @@ class Api:
             搜索的关键词
         since_time : int
             发帖时间截止时间
-        until_time : int
+        until_time : int0
             发帖时间起始时间
         count : int
             搜索的数量
@@ -675,7 +675,7 @@ class Api:
             params["page"] += 1
         return users
 
-    # 采集用户的关注列表
+    # 采集用户的粉丝列表
     def get_user_followers(
             self,
             user_id: str = "",
@@ -698,7 +698,7 @@ class Api:
         Returns
         -------
         list
-            用户的关注列表
+            用户的粉丝列表
 
         References
         ----------
@@ -726,7 +726,7 @@ class Api:
             cursor = data["next_cursor_str"] if "next_cursor_str" in data else 0
         return follower_list
 
-    # 采集用户的粉丝列表
+    # 采集用户的关注列表
     def get_user_followings(
             self,
             user_id: str = "",
@@ -777,6 +777,108 @@ class Api:
             cursor = data["next_cursor_str"] if "next_cursor_str" in data else 0
         return following_list
 
+    # 采集用户的粉丝列表
+    def get_user_followers_ids(
+            self,
+            user_id: str = "",
+            screen_name: str = "",
+            count: int = 200,
+            cursor: str = "",
+    ):
+        """
+        Parameters
+        ----------
+        user_id : str
+            用户id
+        screen_name : str
+            用户显示名
+        count : int
+            采集数量
+        cursor : str
+            中断游标，可用于续采
+
+        Returns
+        -------
+        list
+            用户的粉丝列表
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/v1/accounts-and-users/follow-search-get-users/api-reference/get-followers-ids
+        """
+        if not user_id and not screen_name:
+            raise TwitterError(f"user_id and screen_name are both null")
+        follower_list = []
+        params = self.get_params({"count": count, "stringify_ids": True})
+        if user_id:
+            params["user_id"] = user_id
+        elif screen_name:
+            params["screen_name"] = screen_name
+        while len(follower_list) < count:
+            if cursor:
+                params["cursor"] = cursor
+            data = self.req_twitter(
+                url=self.URI + "/1.1/followers/ids.json", params=params
+            )
+            # 如果没有新数据了 直接返回
+            if len(data["ids"]) <= 1:
+                break
+            follower_list = follower_list + data["ids"]
+            # 获取一下下一页的游标
+            cursor = data["next_cursor_str"] if "next_cursor_str" in data else 0
+        return follower_list
+
+    # 采集用户的关注列表的id
+    def get_user_followings_ids(
+            self,
+            user_id: str = "",
+            screen_name: str = "",
+            count: int = 5000,
+            cursor: str = "",
+    ):
+        """
+        Parameters
+        ----------
+        user_id : str
+            用户id
+        screen_name : str
+            用户显示名
+        count : int
+            采集数量
+        cursor : str
+            中断游标，可用于续采
+
+        Returns
+        -------
+        list
+            用户的关注列表id
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/v1/accounts-and-users/follow-search-get-users/api-reference/get-friends-ids
+        """
+        if not user_id and not screen_name:
+            raise TwitterError(f"user_id and screen_name are both null")
+        following_list = []
+        params = self.get_params({"count": count, "stringify_ids": True})
+        if user_id:
+            params["user_id"] = user_id
+        elif screen_name:
+            params["screen_name"] = screen_name
+        while len(following_list) < count:
+            if cursor:
+                params["cursor"] = cursor
+            data = self.req_twitter(
+                url=self.URI + "/1.1/friends/ids.json", params=params
+            )
+            # 如果没有新数据了 直接返回
+            if len(data["ids"]) <= 1:
+                break
+            following_list = following_list + data["ids"]
+            # 获取一下下一页的游标
+            cursor = data["next_cursor_str"] if "next_cursor_str" in data else 0
+        return following_list
+
     # 采集用户加入的list
     def get_user_list(
             self,
@@ -854,6 +956,25 @@ class Api:
         )
         return data
 
+    # 采集推文相关用户：点赞的人、转发的人、评论的人
+    def get_tweet_related_user(self, post_id: str):
+        """
+        Parameters
+        ----------
+        post_id : str
+            帖子ID
+
+        Returns
+        -------
+        list
+            相关用户列表
+        """
+        params = self.get_params({})
+        data = self.req_twitter(
+            url=self.URI + f"/1.1/statuses/{post_id}/activity/summary.json", params=params
+        )
+        return data
+
 
 class SingletonTWV2Api:
     _instance = None
@@ -876,20 +997,24 @@ def get_api(proxies):
 
 if __name__ == "__main__":
     proxies = {
-        "https": '127.0.0.1:1087',
-        "http": '127.0.0.1:1087'
+        "https": '172.16.7.14:13533',
+        "http": '172.16.7.14:13533'
     }
     api = get_api(proxies)
-    # result = api.get_user_timeline(screen_name='pantwtwtw', count=100)
-    # result = api.get_tweet_comments(post_id="1467435680968105990", count=10000)
+    # result = api.get_user_timeline(screen_name='wwwenwww',count=10)
+    # result = api.get_tweet_comments(post_id="1569953613073764352", count=10000)
     # result = api.get_tweet_info(post_id="1120206463438188544")
-    # result = api.get_tweet_info(post_id="1518852739421323265")
+    # result = api.get_tweet_comments(post_id="1518852739421323265")
     # result = api.get_user_info(screen_name='realllkk520')
     # result = api.get_batch_tweet_info(post_ids=['1417730882090135552'])
     # result = api.get_batch_user_info(user_ids=['1016499781500170240'], screen_names=['bbcchinese'])
-    # result = api.search_tweets(keyword="vivo gimbal camera", count=50)
+    # result = api.search_tweets(keyword="粉 near:25.05,121.50 within:200km", count=10)
     # result = api.search_users(keyword='vivo camera', count=50)
     # result = api.get_user_followers(screen_name='bbcchinese', count=401)
     # result = api.get_user_followings(screen_name='bbcchinese', count=401)
     # result = api.get_user_list(screen_name='harrytemp2', count=401)
-    result = api.get_conversion(post_id='1521574200183566338')
+    # result = api.get_conversion(post_id='1521574200183566338')
+    # result = api.get_tweet_related_user(post_id='1521574200183566338')
+    result = api.get_user_followings_ids(user_id='1653526982')
+    # with open('nathanlawkc.json', 'w', encoding='utf-8') as fp:
+    #     fp.write(json.dumps(result, ensure_ascii=False, indent=4))
